@@ -384,7 +384,6 @@ external unsafe_pcre_exec :
   pos : int ->
   subj_start : int ->
   subj : string ->
-  subgroups2 : int ->
   int array ->
   callout option ->
   unit = "pcre_exec_stub_bc" "pcre_exec_stub"
@@ -398,9 +397,8 @@ let pcre_exec ?(iflags = 0) ?flags ?(rex = def_rex) ?pat ?(pos = 0)
               ?callout subj =
   let rex = match pat with Some str -> regexp str | _ -> rex in
   let iflags = match flags with Some flags -> rflags flags | _ -> iflags in
-  let subgroups2, ovector = make_ovector rex in
-  unsafe_pcre_exec
-    iflags rex ~pos ~subj_start:0 ~subj ~subgroups2 ovector callout;
+  let _, ovector = make_ovector rex in
+  unsafe_pcre_exec iflags rex ~pos ~subj_start:0 ~subj ovector callout;
   ovector
 
 let exec ?iflags ?flags ?rex ?pat ?pos ?callout subj =
@@ -581,7 +579,7 @@ let replace ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
       try
         unsafe_pcre_exec
           iflags rex ~pos:cur_pos ~subj_start:0 ~subj
-          ~subgroups2 ovector callout;
+          ovector callout;
         false
       with Not_found -> true
     then
@@ -625,14 +623,13 @@ let qreplace ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
   let subj_len = String.length subj in
   if pos < 0 || pos > subj_len then invalid_arg "Pcre.qreplace: illegal offset";
   let templ_len = String.length templ in
-  let subgroups2, ovector = make_ovector rex in
+  let _, ovector = make_ovector rex in
   let rec loop full_len subst_lst cur_pos =
     if
       cur_pos > subj_len ||
       try
         unsafe_pcre_exec
-          iflags rex ~pos:cur_pos ~subj_start:0 ~subj
-          ~subgroups2 ovector callout;
+          iflags rex ~pos:cur_pos ~subj_start:0 ~subj ovector callout;
         false
       with Not_found -> true
     then
@@ -674,14 +671,13 @@ let substitute_substrings ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
   let iflags = match flags with Some flags -> rflags flags | _ -> iflags in
   let subj_len = String.length subj in
   if pos < 0 || pos > subj_len then invalid_arg "Pcre.substitute: illegal offset";
-  let subgroups2, ovector = make_ovector rex in
+  let _, ovector = make_ovector rex in
   let rec loop full_len subst_lst cur_pos =
     if
       cur_pos > subj_len ||
       try
         unsafe_pcre_exec
-          iflags rex ~pos:cur_pos ~subj_start:0 ~subj
-          ~subgroups2 ovector callout;
+          iflags rex ~pos:cur_pos ~subj_start:0 ~subj ovector callout;
         false
       with Not_found -> true
     then
@@ -736,9 +732,7 @@ let replace_first ?(iflags = 0) ?flags ?(rex = def_rex) ?pat ?(pos = 0)
     failwith "Pcre.replace_first: backreference denotes nonexistent subpattern";
   if with_lp && nsubs = 0 then failwith "Pcre.replace_first: no backreferences";
   try
-    unsafe_pcre_exec
-      iflags rex ~pos ~subj_start:0 ~subj
-      ~subgroups2 ovector callout;
+    unsafe_pcre_exec iflags rex ~pos ~subj_start:0 ~subj ovector callout;
     let res_len, trans_lst =
       calc_trans_lst subgroups2 ovector subj templ subst_lst in
     let first = Array.unsafe_get ovector 0 in
@@ -757,11 +751,9 @@ let qreplace_first ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
                    ?(pos = 0) ?(templ = "") ?callout subj =
   let rex = match pat with Some str -> regexp str | _ -> rex in
   let iflags = match flags with Some flags -> rflags flags | _ -> iflags in
-  let subgroups2, ovector = make_ovector rex in
+  let _, ovector = make_ovector rex in
   try
-    unsafe_pcre_exec
-      iflags rex ~pos ~subj_start:0 ~subj
-      ~subgroups2 ovector callout;
+    unsafe_pcre_exec iflags rex ~pos ~subj_start:0 ~subj ovector callout;
     let first = Array.unsafe_get ovector 0 in
     let last = Array.unsafe_get ovector 1 in
     let len = String.length templ in
@@ -778,11 +770,9 @@ let substitute_substrings_first ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
                                 ?(pos = 0) ?callout ~subst subj =
   let rex = match pat with Some str -> regexp str | _ -> rex in
   let iflags = match flags with Some flags -> rflags flags | _ -> iflags in
-  let subgroups2, ovector = make_ovector rex in
+  let _, ovector = make_ovector rex in
   try
-    unsafe_pcre_exec
-      iflags rex ~pos ~subj_start:0 ~subj
-      ~subgroups2 ovector callout;
+    unsafe_pcre_exec iflags rex ~pos ~subj_start:0 ~subj ovector callout;
     let subj_len = String.length subj in
     let prefix_len = Array.unsafe_get ovector 0 in
     let last = Array.unsafe_get ovector 1 in
@@ -841,8 +831,7 @@ let internal_psplit flags rex max pos callout subj =
           if prematch &&
             try
               unsafe_pcre_exec
-                flags rex ~pos ~subj_start:pos ~subj
-                ~subgroups2 ovector callout;
+                flags rex ~pos ~subj_start:pos ~subj ovector callout;
               true
             with Not_found -> false
           then
@@ -856,8 +845,7 @@ let internal_psplit flags rex max pos callout subj =
           if
             try
               unsafe_pcre_exec
-                flags rex ~pos ~subj_start:pos ~subj
-                ~subgroups2 ovector callout;
+                flags rex ~pos ~subj_start:pos ~subj ovector callout;
               false
             with Not_found -> true
           then string_unsafe_sub subj pos len :: strs
@@ -872,7 +860,7 @@ let internal_psplit flags rex max pos callout subj =
                   try
                     unsafe_pcre_exec
                       (flags lor 0x0410) rex ~pos ~subj_start:pos ~subj
-                      ~subgroups2 ovector callout;
+                      ovector callout;
                     true
                   with Not_found -> false
                 then
@@ -964,8 +952,7 @@ let full_split ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
           if prematch &&
             try
               unsafe_pcre_exec
-                iflags rex ~pos ~subj_start:pos ~subj
-                ~subgroups2 ovector callout;
+                iflags rex ~pos ~subj_start:pos ~subj ovector callout;
                true
             with Not_found -> false
           then
@@ -983,8 +970,7 @@ let full_split ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
           if
             try
               unsafe_pcre_exec
-                iflags rex ~pos ~subj_start:pos ~subj
-                ~subgroups2 ovector callout;
+                iflags rex ~pos ~subj_start:pos ~subj ovector callout;
               false
             with Not_found -> true
           then
@@ -1002,7 +988,7 @@ let full_split ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
                     try
                       unsafe_pcre_exec
                         (iflags lor 0x0410) rex ~pos ~subj_start:pos ~subj
-                        ~subgroups2 ovector callout;
+                        ovector callout;
                       true
                     with Not_found -> false
                   then
