@@ -70,7 +70,7 @@ typedef const unsigned char *chartables;  /* Type of chartable sets */
 
 /* Contents of callout data */
 struct cod {
-  long subj_start;         /* Start of subject string */
+  long subj_start;        /* Start of subject string */
   value *v_substrings_p;  /* Pointer to substrings matched so far */
   value *v_cof_p;         /* Pointer to callout function */
   value v_exn;            /* Possible exception raised by callout function */
@@ -187,7 +187,12 @@ static void pcre_dealloc_regexp(value v_rex)
 {
   void *extra = (void *) Field(v_rex, 2);
   (pcre_free)((void *) Field(v_rex, 1));
-  if (extra != NULL) (pcre_free)(extra);
+  if (extra != NULL)
+#ifdef PCRE_STUDY_JIT_COMPILE
+    pcre_free_study(extra);
+#else
+    pcre_free(extra);
+#endif
 }
 
 /* Makes OCaml-string from PCRE-version */
@@ -267,10 +272,10 @@ CAMLprim value pcre_compile_stub(value v_opt, value v_tables, value v_pat)
      could not be compiled */
   if (regexp == NULL) raise_bad_pattern(error, error_ofs);
 
-  /* Finalized value: GC will do a full cycle every 500 regexp allocations
-     (one regexp consumes in average probably less than 100 bytes ->
-     maximum of 50000 bytes unreclaimed regexps) */
-  v_rex = caml_alloc_final(4, pcre_dealloc_regexp, 100, 50000);
+  /* GC will do a full cycle every 1_000_000 regexp allocations (a typical
+     regexp probably consumes less than 100 bytes -> maximum of 100_000_000
+     bytes unreclaimed regexps) */
+  v_rex = caml_alloc_final(4, pcre_dealloc_regexp, 1, 1000000);
 
   /* Field[1]: compiled regular expression (Field[0] is finalizing
      function! See above!) */
@@ -306,7 +311,7 @@ CAMLprim value pcre_set_imp_match_limit_recursion_stub(value v_rex, value v_lim)
 {
   pcre_extra *extra = (pcre_extra *) Field(v_rex, 2);
   if (extra == NULL) {
-    extra = caml_stat_alloc(sizeof(pcre_extra));
+    extra = pcre_malloc(sizeof(pcre_extra));
     extra->flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
     Field(v_rex, 2) = (value) extra;
   } else {
@@ -336,7 +341,7 @@ CAMLprim value pcre_set_imp_match_limit_stub(value v_rex, value v_lim)
 {
   pcre_extra *extra = (pcre_extra *) Field(v_rex, 2);
   if (extra == NULL) {
-    extra = caml_stat_alloc(sizeof(pcre_extra));
+    extra = pcre_malloc(sizeof(pcre_extra));
     extra->flags = PCRE_EXTRA_MATCH_LIMIT;
     Field(v_rex, 2) = (value) extra;
   } else {
@@ -496,19 +501,33 @@ static inline void handle_exec_error(char *loc, const int ret)
   }
 }
 
+static inline void handle_pcre_exec_result(
+  int *ovec, value v_ovec, long ovec_len, long subj_start, int ret)
+{
+  ovec_dst_ptr ocaml_ovec = (ovec_dst_ptr) &Field(v_ovec, 0);
+  const int subgroups2 = ret * 2;
+  const int subgroups2_1 = subgroups2 - 1;
+  const int *ovec_src = ovec + subgroups2_1;
+  ovec_dst_ptr ovec_clear_stop = ocaml_ovec + (ovec_len * 2) / 3;
+  ovec_dst_ptr ovec_dst = ocaml_ovec + subgroups2_1;
+  copy_ovector(subj_start, ovec_src, ovec_dst, subgroups2);
+  while (++ovec_dst < ovec_clear_stop) *ovec_dst = -1;
+}
+
 /* Executes a pattern match with runtime options, a regular expression, a
    matching position, the start of the the subject string, a subject string,
    a number of subgroup offsets, an offset vector and an optional callout
    function */
 CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
                               value v_subj_start, value v_subj,
-                              value v_subgroups2, value v_ovec,
-                              value v_maybe_cof)
+                              value v_ovec, value v_maybe_cof)
 {
+  int ret;
   long
     pos = Long_val(v_pos),
     len = caml_string_length(v_subj),
     subj_start = Long_val(v_subj_start);
+  long ovec_len = Wosize_val(v_ovec);
 
   if (pos > len || pos < subj_start)
     caml_invalid_argument("Pcre.pcre_exec_stub: illegal position");
@@ -525,24 +544,16 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
     const char *ocaml_subj =
       String_val(v_subj) + subj_start;  /* Subject string */
     const int opt = Int_val(v_opt);  /* Runtime options */
-    int subgroups2 = Int_val(v_subgroups2);
-    const int subgroups2_1 = subgroups2 - 1;
-    const int subgroups3 = (subgroups2 >> 1) + subgroups2;
 
     /* Special case when no callout functions specified */
     if (v_maybe_cof == None) {
       int *ovec = (int *) &Field(v_ovec, 0);
 
       /* Performs the match */
-      const int ret =
-        pcre_exec(code, extra, ocaml_subj, len, pos, opt, ovec, subgroups3);
+      ret = pcre_exec(code, extra, ocaml_subj, len, pos, opt, ovec, ovec_len);
 
       if (ret < 0) handle_exec_error("pcre_exec_stub", ret);
-      else {
-        const int *ovec_src = ovec + subgroups2_1;
-        ovec_dst_ptr ovec_dst = (ovec_dst_ptr) ovec + subgroups2_1;
-        copy_ovector(subj_start, ovec_src, ovec_dst, subgroups2);
-      }
+      else handle_pcre_exec_result(ovec, v_ovec, ovec_len, subj_start, ret);
     }
 
     /* There are callout functions */
@@ -550,8 +561,7 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
       value v_cof = Field(v_maybe_cof, 0);
       value v_substrings;
       char *subj = caml_stat_alloc(sizeof(char) * len);
-      int *ovec = caml_stat_alloc(sizeof(int) * subgroups3);
-      int ret;
+      int *ovec = caml_stat_alloc(sizeof(int) * ovec_len);
       struct cod cod = { 0, (value *) NULL, (value *) NULL, (value) NULL };
       struct pcre_extra new_extra =
 #ifdef PCRE_EXTRA_MATCH_LIMIT_RECURSION
@@ -585,7 +595,7 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
 
         if (extra == NULL) {
           ret = pcre_exec(code, &new_extra, subj, len, pos, opt, ovec,
-                          subgroups3);
+                          ovec_len);
         }
         else {
           new_extra.flags = PCRE_EXTRA_CALLOUT_DATA | extra->flags;
@@ -597,7 +607,7 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
 #endif
 
           ret = pcre_exec(code, &new_extra, subj, len, pos, opt, ovec,
-                          subgroups3);
+                          ovec_len);
         }
 
         caml_stat_free(subj);
@@ -608,9 +618,7 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
         if (ret == PCRE_ERROR_CALLOUT) caml_raise(cod.v_exn);
         else handle_exec_error("pcre_exec_stub(callout)", ret);
       } else {
-        int *ovec_src = ovec + subgroups2_1;
-        ovec_dst_ptr ovec_dst = &Field(v_ovec, 0) + subgroups2_1;
-        copy_ovector(subj_start, ovec_src, ovec_dst, subgroups2);
+        handle_pcre_exec_result(ovec, v_ovec, ovec_len, subj_start, ret);
         caml_stat_free(ovec);
       }
     }
@@ -624,17 +632,17 @@ CAMLprim value pcre_exec_stub(value v_opt, value v_rex, value v_pos,
 CAMLprim value pcre_exec_stub_bc(value *argv, int __unused argn)
 {
   return pcre_exec_stub(argv[0], argv[1], argv[2], argv[3],
-                        argv[4], argv[5], argv[6], argv[7]);
+                        argv[4], argv[5], argv[6]);
 }
 
 /* Generates a new set of chartables for the current locale (see man
    page of PCRE */
 CAMLprim value pcre_maketables_stub(value __unused v_unit)
 {
-  /* GC will do a full cycle every 100 table set allocations
-     (one table set consumes 864 bytes -> maximum of 86400 bytes
-     unreclaimed table sets) */
-  const value v_res = caml_alloc_final(2, pcre_dealloc_tables, 864, 86400);
+  /* GC will do a full cycle every 1_000_000 table set allocations (one
+     table set consumes 864 bytes -> maximum of 864_000_000 bytes unreclaimed
+     table sets) */
+  const value v_res = caml_alloc_final(2, pcre_dealloc_tables, 1, 1000000);
   Field(v_res, 1) = (value) pcre_maketables();
   return v_res;
 }
